@@ -9,7 +9,6 @@ import os
 import locale
 
 def run(bk):
-
     # get current epub version
     epubversion = "2.0"
     if bk.launcher_version() >= 20160102:
@@ -17,19 +16,39 @@ def run(bk):
 
     errors = 0
 
+    print(" ")
     # Verify Python sys.path is set correctly for your installation
     print("Verify sys.path settings manually")
     for apath in sys.path:
         print(" ", apath)
-    
+
     # verify if all the proper external python packages are available
     # as documented in our Python Developer's Framework as being available
     print(" ")
     print("Verifying proper Python packages are available")
+
+    print("  Checking GUI modules")
+    if bk.launcher_version() >= 20170227:
+        try:
+            print("    Python Package: PyQt5", end=" ")
+            from PyQt5 import QtWidgets
+            print("(%s) Found"%QtWidgets.__file__)
+        except ImportError:
+            print("Missing")
+            errors += 1
+
+    try:
+        print("    Python Package: tkinter", end=" ")
+        import tkinter
+        print("(%s) Found"%tkinter.__file__)
+    except ImportError:
+        print("Missing")
+        errors += 1
+
     try:
         print("  Python Package: PIL", end=" ")
         import PIL
-        print("Found")
+        print("(%s) Found"%PIL.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -37,15 +56,20 @@ def run(bk):
     try:
         print("  Python Package: cssselect", end=" ")
         import cssselect
-        print("Found")
+        print("(%s) Found"%cssselect.__file__)
     except ImportError:
         print("Missing")
         errors += 1
 
     try:
-        print("  Python Package: cssutils", end=" ")
-        import cssutils
-        print("Found")
+    	if bk.launcher_version() < 20190826:
+            print("  Python Package: cssutils", end=" ")
+            import cssutils
+            print("(%s) Found"%cssutils.__file__)
+    	else:
+            print("  Python Package: css-parser", end=" ")
+            import css_parser
+            print("(%s) Found"%css_parser.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -53,7 +77,7 @@ def run(bk):
     try:
         print("  Python Package: html5lib", end=" ")
         import html5lib
-        print("Found")
+        print("(%s) Found"%html5lib.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -61,7 +85,7 @@ def run(bk):
     try:
         print("  Python Package: lxml", end=" ")
         import lxml
-        print("Found")
+        print("(%s) Found"%lxml.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -69,7 +93,7 @@ def run(bk):
     try:
         print("  Python Package: regex", end=" ")
         import regex
-        print("Found")
+        print("(%s) Found"%regex.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -77,7 +101,7 @@ def run(bk):
     try:
         print("  Python Package: chardet", end=" ")
         import chardet
-        print("Found")
+        print("(%s) Found"%chardet.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -85,7 +109,7 @@ def run(bk):
     try:
         print("  Python Package: six", end=" ")
         import six
-        print("Found")
+        print("(%s) Found"%six.__file__)
     except ImportError:
         print("Missing")
         errors += 1
@@ -230,14 +254,16 @@ def run(bk):
 
     # all files in the OPF manifest
     working = False
-    print("  bk.manifest_iter() works", end=" ")
+    print("  Listing manifested files...")
     for (id, href, mime) in bk.manifest_iter():
+        print("    %s" % href)
         if epubversion.startswith("3"):
             working = href in ("Text/Section0001.xhtml", "toc.ncx", "Text/nav.xhtml", "Styles/sgc-nav.css")
         else:
             working = href in ("Text/Section0001.xhtml", "toc.ncx")
         if not working:
-            break;
+            break
+    print("  bk.manifest_iter() works", end=" ")
     print(working)
     if not working:
         errors += 1
@@ -272,7 +298,7 @@ def run(bk):
     print(working)
     if not working:
         errors += 1
-    
+
     # Example of reading a file with a specific OPF manifest id
     working = False
     print("  reading a specific OPF manifest id works", end=" ")
@@ -319,7 +345,7 @@ def run(bk):
     print(working)
     if not working:
         errors += 1
-    
+
     # Examples of Using the Convenience Functions
     working = False
     print("  basename to id mapping works", end=" ")
@@ -345,17 +371,17 @@ def run(bk):
     else:
         print("Success - All Tests of Plugin Operations Passed")
 
+
     # Setting the proper Return value is important.
     # 0 - means success
     # anything else means failure
     return 0
- 
- 
- 
+
 def main():
     print("I reached main when I should not have\n")
     return -1
-    
+
+
 if __name__ == "__main__":
     sys.exit(main())
 
