@@ -1,8 +1,8 @@
 /************************************************************************
 **
-**  Copyright (C) 2017-2019  Kevin B, Hendricks, Stratford, ON Canada
-**  Copyright (C) 2012, 2013 John Schember <john@nachtimwald.com>
-**  Copyright (C) 2012, 2013 Dave Heiland
+**  Copyright (C) 2015-2020 Kevin B, Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2012-2013 John Schember <john@nachtimwald.com>
+**  Copyright (C) 2012-2013 Dave Heiland
 **
 **  This file is part of Sigil.
 **
@@ -22,7 +22,9 @@
 *************************************************************************/
 
 #include <QApplication>
+#include <QGuiApplication>
 #include <QtCore/QFileInfo>
+#include <QEventLoop>
 #include <QImage>
 #include <QPixmap>
 #include <QtWidgets/QLayout>
@@ -30,9 +32,10 @@
 #include <QtWebEngineWidgets/QWebEngineSettings>
 
 #include "MainUI/MainWindow.h"
-#include "Dialogs/SelectFiles.h"
 #include "Misc/SettingsStore.h"
 #include "sigil_constants.h"
+#include "ViewEditors/SimplePage.h"
+#include "Dialogs/SelectFiles.h"
 
 static const int COL_NAME = 0;
 static const int COL_IMAGE = 1;
@@ -49,8 +52,9 @@ const QString IMAGE_HTML_BASE_PREVIEW =
     "body { -webkit-user-select: none; }"
     "img { display: block; margin-left: auto; margin-right: auto; border-style: solid; border-width: 1px; max-width: 95%; max-height: 95%}"
     "</style>"
+    "</head>"
     "<body>"
-    "<div><img src=\"%2\" /></div>"
+    "<div><img src=\"%1\" /></div>"
     "</body>"
     "</html>";
 
@@ -67,7 +71,7 @@ SelectFiles::SelectFiles(QString title, QList<Resource *> media_resources, QStri
 {
     ui.setupUi(this);
     setWindowTitle(title);
-
+    m_WebView->setPage(new SimplePage(m_WebView));
     m_WebView->setContextMenuPolicy(Qt::NoContextMenu);
     m_WebView->setFocusPolicy(Qt::NoFocus);
     m_WebView->setAcceptDrops(false);
@@ -121,7 +125,11 @@ QStringList SelectFiles::SelectedImages()
 void SelectFiles::SetImages()
 {
     ui.Details->clear();
-    m_WebView->setHtml("", QUrl());
+    QString html = "<html><head><title></title></head><body></body></html>";
+    if (Utility::IsDarkMode()) {
+        html = Utility::AddDarkCSS(html);
+    }
+    m_WebView->setHtml(html, QUrl());
 
     m_SelectFilesModel->clear();
     QStringList header;
@@ -148,11 +156,11 @@ void SelectFiles::SetImages()
             continue;
         }
 
-        QString filepath = "../" + resource->GetRelativePathToOEBPS();
+        QString filepath = resource->GetRelativePath();
         QList<QStandardItem *> rowItems;
         QStandardItem *name_item = new QStandardItem();
-        name_item->setText(resource->Filename());
-        name_item->setToolTip(filepath);
+        name_item->setText(filepath);
+        name_item->setToolTip(resource->ShortPathName());
         name_item->setData(static_cast<int>(type), Qt::UserRole);
         name_item->setData(resource->GetFullPath(), Qt::UserRole + 1);
         name_item->setEditable(false);
@@ -259,7 +267,11 @@ void SelectFiles::SetPreviewImage()
     QStandardItem *item = GetLastSelectedImageItem();
     
     ui.Details->clear();
-    m_WebView->setHtml("", QUrl());
+    QString html = "<html><head><title></title></head><body></body></html>";
+    if (Utility::IsDarkMode()) {
+        html = Utility::AddDarkCSS(html);
+    }
+    m_WebView->setHtml(html, QUrl());
 
     if (!item || item->text().isEmpty()) {
         m_PreviewReady = true;
@@ -297,9 +309,13 @@ void SelectFiles::SetPreviewImage()
         details = QString("%2x%3px | %4 KB | %5%6").arg(img.width()).arg(img.height())
                   .arg(fsize).arg(grayscale_color).arg(colorsInfo);
 
-        MainWindow::clearMemoryCaches();
+        // MainWindow::clearMemoryCaches();
         const QUrl resourceUrl = QUrl::fromLocalFile(path);
         QString html = IMAGE_HTML_BASE_PREVIEW.arg(resourceUrl.toString());
+	if (Utility::IsDarkMode()) {
+            html = Utility::AddDarkCSS(html);
+	}
+        m_WebView->page()->setBackgroundColor(Utility::WebViewBackgroundColor());
         m_PreviewLoaded = false;
         m_WebView->setHtml(html, resourceUrl);
         loading_resources = true;
@@ -311,14 +327,23 @@ void SelectFiles::SetPreviewImage()
         MainWindow::clearMemoryCaches();
         html = VIDEO_HTML_BASE.arg(resourceUrl.toString());
         m_PreviewLoaded = false;
+	if (Utility::IsDarkMode()) {
+            html = Utility::AddDarkCSS(html);
+	}
+        m_WebView->page()->setBackgroundColor(Utility::WebViewBackgroundColor());
+        m_PreviewLoaded = false;
         m_WebView->setHtml(html, resourceUrl);
         loading_resources = true;
         details = QString("%1 MB").arg(fmbsize);
     } else if (resource_type == Resource::AudioResourceType) {
         QString html;
         const QUrl resourceUrl = QUrl::fromLocalFile(path);
-        MainWindow::clearMemoryCaches();
+        // MainWindow::clearMemoryCaches();
         html = AUDIO_HTML_BASE.arg(resourceUrl.toString());
+	if (Utility::IsDarkMode()) {
+            html = Utility::AddDarkCSS(html);
+	}
+        m_WebView->page()->setBackgroundColor(Utility::WebViewBackgroundColor());
         m_PreviewLoaded = false;
         m_WebView->setHtml(html, resourceUrl);
         loading_resources = true;
@@ -329,7 +354,7 @@ void SelectFiles::SetPreviewImage()
     // because setHtml loads external resources asynchronously
     if (loading_resources) {
         while(!IsPreviewLoaded()) {
-            qApp->processEvents();
+            qApp->processEvents(QEventLoop::ExcludeUserInputEvents);
         }
     }
     ui.Details->setText(details);
