@@ -1,6 +1,7 @@
 /************************************************************************
 **
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2015-2019 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -24,6 +25,7 @@
 #define BOOKBROWSER_H
 
 #include <QtCore/QSharedPointer>
+#include <QWidget>
 #include <QtWidgets/QDockWidget>
 
 #include "BookManipulation/Book.h"
@@ -34,6 +36,7 @@ class HTMLResource;
 class ImageResource;
 class OPFModel;
 class Resource;
+class CSSResource;
 class QAction;
 class QMenu;
 class QModelIndex;
@@ -115,8 +118,18 @@ public:
      */
     void UpdateSelection(Resource *resource);
 
-    void AddFile(QString filepath);
+    Resource* AddFile(QString filepath);
 
+    /**
+     *  Allow automatic renaming of resources
+     */
+    void RenameResourceList(const QList<Resource *> &resources, const QStringList &newfilenames);
+
+    /**
+     *  Allow automatic moving of resources
+     */
+    void MoveResourceList(const QList<Resource *> &resources, const QStringList &newbookpaths);
+ 
 public slots:
 
     /**
@@ -161,8 +174,8 @@ public slots:
     void AddNewHTML();
     void AddNewCSS();
     void AddNewSVG();
-    void CreateHTMLTOCCSSFile();
-    void CreateIndexCSSFile();
+    CSSResource* CreateHTMLTOCCSSFile();
+    CSSResource* CreateIndexCSSFile();
     /**
      * Implements the Add Existing context menu action functionality.
      */
@@ -182,6 +195,13 @@ public slots:
      */
     void SelectRenamedResource();
 
+    /**
+     * Updates the selection in the book display
+     * to the one resource being moved
+     * and resets the focus to Book Browser.
+     */
+    void SelectMovedResource();
+
     void SaveAsUrl(const QUrl &url);
 
 signals:
@@ -217,6 +237,9 @@ signals:
 
     void ResourcesDeleted();
 
+    void ResourcesMoved();
+
+
     /**
      * Wired to the current MainWindow::UpdateBrowserSelectionToTab signal.
      */
@@ -255,7 +278,7 @@ private slots:
     void SaveAsFile(Resource *resource);
     void SaveAsFiles();
 
-    void OpenWith() const;
+    void OpenWith();
 
     void OpenWithEditor(int) const;
 
@@ -264,6 +287,17 @@ private slots:
      */
     void Rename();
 
+    /**
+     * Implements the Regular Expression Rename context menu action functionality.
+     */
+    void REXRename();
+
+    /**
+     * Implements the Move context menu action functionality.
+     */
+    void Move();
+
+
     QString GetFirstAvailableTemplateName(QString base, QString number_string);
 
     /**
@@ -271,6 +305,11 @@ private slots:
      */
     void RenameSelected();
 
+    /**
+     * Implements the Move selected context menu action functionality.
+     */
+    void MoveSelected();
+
     /**
      * Implements the Delete context menu action functionality.
      */
@@ -455,6 +494,8 @@ private:
     QAction *m_AddNewSVG;
     QAction *m_AddExisting;
     QAction *m_Rename;
+    QAction *m_RERename;
+    QAction *m_Move;
     QAction *m_Delete;
     QAction *m_Merge;
     QAction *m_MergeWithPrevious;
@@ -496,6 +537,7 @@ private:
     QList <QModelIndex> m_SavedSelection;
 
     Resource *m_RenamedResource;
+    Resource *m_MovedResource;
 };
 
 #endif // BOOKBROWSER_H
