@@ -1,6 +1,6 @@
 /************************************************************************
 **
-**  Copyright (C) 2016-2019 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2015-2019 Kevin B. Hendricks, Stratford, Ontario, Canada
 **  Copyright (C) 2012-2015 John Schember <john@nachtimwald.com>
 **  Copyright (C) 2012 Dave Heiland
 **  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
@@ -57,6 +57,7 @@ class QLabel;
 class QSignalMapper;
 class QSlider;
 class QTimer;
+class QActionGroup;
 class FindReplace;
 class TabManager;
 class BookBrowser;
@@ -101,9 +102,18 @@ public:
      * @param parent The window's parent object.
      * @param flags The flags used to modify window behavior.
      */
-    MainWindow(const QString &openfilepath = QString(), bool is_internal = false, QWidget *parent = 0, Qt::WindowFlags flags = 0);
+    MainWindow(const QString &openfilepath = QString(), 
+	       const QString version = QString(),
+	       bool is_internal = false, 
+	       QWidget *parent = 0, 
+	       Qt::WindowFlags flags = 0);
     ~MainWindow();
 
+    void maybe_fixup_dockwidget_geometry(QDockWidget * widget);
+
+    // returns true if MainWindow is Maximized or is FullScreen
+    bool isMaxOrFull();
+
     /**
      * The book currently being edited.
      *
@@ -154,7 +164,7 @@ public:
      * The location of the last bookmark.
      */
     struct LocationBookmark {
-        QString filename;
+        QString bookpath;
         QString bv_caret_location_update;
         int cv_cursor_position;
     };
@@ -193,7 +203,8 @@ public:
 
     QString GetCurrentFilePath();
 
-
+    void DebugCurrentWidgetSizes();
+    
 public slots:
 
     void OpenUrl(const QUrl &url);
@@ -227,10 +238,14 @@ public slots:
 
     void runPlugin(QAction *action);
 
-    void ResourcesAddedOrDeleted();
+    void ResourcesAddedOrDeletedOrMoved();
 
     void launchExternalXEditor();
 
+    void StandardizeEpub();
+
+    void CreateEpubLayout();
+
 signals:
     void SettingsChanged();
 
@@ -256,12 +271,20 @@ protected:
 
 private slots:
 
+    void UpdateLastSizes();
+
+    void RestoreLastNormalGeometry();
+
     void AddCover();
 
+
     /**
      * Implements New action functionality.
      */
-    void New();
+    void New(const QString version = QString());
+    void NewDefault();
+    void NewEpub2();
+    void NewEpub3();
 
     /**
      * Implements Open action functionality.
@@ -345,6 +368,8 @@ private slots:
      */
     void ApplicationFocusChanged(QWidget *old, QWidget *now);
 
+    void ApplicationPaletteChanged();
+
     /** 
      *  Quick Launch Plugins via icon button
      */
@@ -360,26 +385,6 @@ private slots:
     void PasteClipEntriesIntoCurrentTarget(const QList<ClipEditorModel::clipEntry *> &clips);
     void PasteClipEntriesIntoPreviousTarget(const QList<ClipEditorModel::clipEntry *> &clips);
     void PasteClipIntoCurrentTarget(int clip_number);
-    void PasteClip1IntoCurrentTarget();
-    void PasteClip2IntoCurrentTarget();
-    void PasteClip3IntoCurrentTarget();
-    void PasteClip4IntoCurrentTarget();
-    void PasteClip5IntoCurrentTarget();
-    void PasteClip6IntoCurrentTarget();
-    void PasteClip7IntoCurrentTarget();
-    void PasteClip8IntoCurrentTarget();
-    void PasteClip9IntoCurrentTarget();
-    void PasteClip10IntoCurrentTarget();
-    void PasteClip11IntoCurrentTarget();
-    void PasteClip12IntoCurrentTarget();
-    void PasteClip13IntoCurrentTarget();
-    void PasteClip14IntoCurrentTarget();
-    void PasteClip15IntoCurrentTarget();
-    void PasteClip16IntoCurrentTarget();
-    void PasteClip17IntoCurrentTarget();
-    void PasteClip18IntoCurrentTarget();
-    void PasteClip19IntoCurrentTarget();
-    void PasteClip20IntoCurrentTarget();
 
     /**
      * Implements the set CodeView functionality.
@@ -596,12 +601,12 @@ private slots:
     void EditTOCDialog();
     void CreateHTMLTOC();
 
-    void ChangeCasing(int casing_mode);
+    void ChangeCasing(QAction* act);
 
     void MarkSelection();
     void ClearMarkedText(ContentTab *old_tab = NULL);
 
-    void ApplyHeadingStyleToTab(const QString &heading_type);
+    void ApplyHeadingStyleToTab(QAction* act);
     void SetPreserveHeadingAttributes(bool new_state);
 
     void GoBackFromLinkOrStyle();
@@ -623,7 +628,7 @@ private slots:
     void DeleteReportsStyles(QList<BookReports::StyleData *> reports_styles_to_delete);
 
     void DeleteFilenames(QStringList files_to_delete);
-    void OpenFile(QString filename, int line = -1);
+    void OpenFile(QString file_shortpathname, int line = -1);
 
     void UpdateClipsUI();
 
@@ -635,7 +640,7 @@ private slots:
 
 private:
     void updateToolTipsOnPluginIcons();
-    void UpdateClipButton(int clip_number, QAction *ui_action);
+    void UpdateClipButton(QAction *ui_action);
     void InsertFiles(const QStringList &selected_images);
     void InsertFilesFromDisk();
 
@@ -665,6 +670,8 @@ private:
      */
     bool MaybeSaveDialogSaysProceed();
 
+    bool ProceedToOverwrite(const QString &msg, const QString &filename );
+
     /**
      * Makes the provided book the current one.
      *
@@ -676,7 +683,7 @@ private:
      * Creates a new, empty book and replaces
      * the current one with it.
      */
-    void CreateNewBook();
+    void CreateNewBook(const QString version=QString(), const QStringList &book_paths=QStringList());
 
     /**
      * Saves the current book to the file specified.
@@ -762,6 +769,11 @@ private:
      */
     void PlatformSpecificTweaks();
 
+    /**
+     * Tweak default UI fonts based on the OS platform.
+     */
+    void SetupUiFont();
+
     /**
      * Extends the UI with extra widgets and tweaks.
      * Qt Designer is not able to create all the widgets
@@ -785,7 +797,7 @@ private:
      *
      * @param openfilepath The path to the file to load. Can be empty.
      */
-    void LoadInitialFile(const QString &openfilepath, bool is_internal = false);
+    void LoadInitialFile(const QString &openfilepath, const QString version=QString(), bool is_internal = false);
 
     /**
      * Connects all the required signals to their slots.
@@ -808,6 +820,9 @@ private:
 
     void SetupPreviewTimer();
 
+    void FixDuplicateFilenames();
+    void MoveContentFilesToStdFolders();
+
     ///////////////////////////////
     // PRIVATE MEMBER VARIABLES
     ///////////////////////////////
@@ -924,9 +939,7 @@ private:
     /**
      * Collects signals and sends specific parameters to the connected slots.
      */
-    QSignalMapper *m_headingMapper;
-    QSignalMapper *m_casingChangeMapper;
-    QSignalMapper *m_pluginMapper;
+    QActionGroup *m_casingChangeGroup;
 
     /**
      * The Search Manager dialog
@@ -967,6 +980,10 @@ private:
      * Workaround for Qt 4.8 bug, to track the last known window size when not maximized.
      */
     QByteArray m_LastWindowSize;
+    QByteArray m_LastState;
+    bool m_FirstTime;
+    bool m_PendingLastSizeUpdate;
+    bool m_SaveLastEnabled;
 
     QTimer m_PreviewTimer;
 
@@ -986,8 +1003,18 @@ private:
 
     QStringList m_pluginList;
     bool m_SaveCSS;
+    bool m_IsClosing;
 
     QList<QAction*> m_qlactions;
+
+    QList<QAction*> m_clactions;
+
+
+    /**
+     * Collects signals and sends specific parameters to the connected slots.
+     */
+    QActionGroup *m_headingActionGroup;
+
     /**
      * Holds all the widgets Qt Designer created for us.
      */
