@@ -1,7 +1,7 @@
 /************************************************************************
 **
-**  Copyright (C) 2019 Kevin B. Hendricks, Stratford, Ontario, Canada
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2015-2019 Kevin B. Hendricks, Stratford, Ontario, Canada
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -37,6 +37,7 @@ ContentTab::ContentTab(Resource *resource, QWidget *parent)
 {
     connect(resource, SIGNAL(Deleted(const Resource *)),          this, SLOT(UnderlyingResourceDeleted()));
     connect(resource, SIGNAL(Renamed(const Resource *, QString)), this, SLOT(EmitTabRenamed()));
+    connect(resource, SIGNAL(Moved(const Resource *, QString)), this, SLOT(EmitTabRenamed()));
     m_Layout->setContentsMargins(0, 0, 0, 0);
     setLayout(m_Layout);
 }
@@ -54,6 +55,12 @@ QString ContentTab::GetFilename()
 }
 
 
+QString ContentTab::GetShortPathName()
+{
+    return m_Resource->ShortPathName();
+}
+
+
 QIcon ContentTab::GetIcon()
 {
     return m_Resource->Icon();
