@@ -1,6 +1,7 @@
 /************************************************************************
 **
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2015-2020 Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2009-2011 Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -20,6 +21,7 @@
 *************************************************************************/
 
 #include <QApplication>
+#include <QGuiApplication>
 #include <QtCore/QDateTime>
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
@@ -30,16 +32,20 @@
 #include <QtGui/QClipboard>
 #include <QtWidgets/QLayout>
 #include <QtWidgets/QMenu>
-#include <QtWebKitWidgets/QWebView>
+#include <QtWebEngineWidgets/QWebEngineView>
+#include <QtWebEngineWidgets/QWebEngineProfile>
 #include <QtPrintSupport/QPrinter>
 #include <QtPrintSupport/QPrintDialog>
 #include <QtPrintSupport/QPrintPreviewDialog>
-
+#include <QDebug>
 
 #include "MainUI/MainWindow.h"
+#include "ViewEditors/SimplePage.h"
 #include "Misc/OpenExternally.h"
 #include "Misc/SettingsStore.h"
+#include "Misc/Utility.h"
 #include "ResourceObjects/ImageResource.h"
+#include "sigil_constants.h"
 #include "Tabs/ImageTab.h"
 
 const QString IMAGE_HTML_BASE =
@@ -51,6 +57,7 @@ const QString IMAGE_HTML_BASE =
     "hr { width: 75%; }"
     "div { text-align: center; }"
     "</style>"
+    "</head>"
     "<body>"
     "<p><img src=\"%1\" /></p>"
     "<hr />"
@@ -58,14 +65,16 @@ const QString IMAGE_HTML_BASE =
     "</body>"
     "</html>";
 
+
 ImageTab::ImageTab(ImageResource *resource, QWidget *parent)
     :
     ContentTab(resource, parent),
-    m_WebView(new QWebView(this)),
+    m_WebView(new QWebEngineView(this)),
     m_ContextMenu(new QMenu(this)),
     m_OpenWithContextMenu(new QMenu(this)),
     m_openWithMapper(new QSignalMapper(this))
 {
+    m_WebView->setPage(new SimplePage(m_WebView));
     m_WebView->setContextMenuPolicy(Qt::CustomContextMenu);
     m_WebView->setFocusPolicy(Qt::NoFocus);
     m_WebView->setAcceptDrops(false);
@@ -167,7 +176,7 @@ void ImageTab::UpdateDisplay()
 
 void ImageTab::RefreshContent()
 {
-    MainWindow::clearMemoryCaches();
+    m_WebView->page()->profile()->clearHttpCache();
     const QString path = m_Resource->GetFullPath();
     const QFileInfo fileInfo = QFileInfo(path);
     const double ffsize = fileInfo.size() / 1024.0;
@@ -184,8 +193,13 @@ void ImageTab::RefreshContent()
         colorsInfo = QString(" %1bpp (%2 %3)").arg(img.bitPlaneCount()).arg(img.colorCount()).arg(colors_shades);
     }
 
-    const QString html = IMAGE_HTML_BASE.arg(imgUrl.toString()).arg(img.width()).arg(img.height()).arg(fsize)
+    QString html = IMAGE_HTML_BASE.arg(imgUrl.toString()).arg(img.width()).arg(img.height()).arg(fsize)
                          .arg(grayscale_color).arg(colorsInfo);
+
+    if (Utility::IsDarkMode()) {
+	html = Utility::AddDarkCSS(html);
+    }
+    m_WebView->page()->setBackgroundColor(Utility::WebViewBackgroundColor());
     m_WebView->setHtml(html, imgUrl);
 }
 
@@ -211,12 +225,12 @@ void ImageTab::openWith()
 
     if (data.isValid()) {
         const QUrl &resourceUrl = data.toUrl();
-        const QString &editorPath = OpenExternally::selectEditorForResourceType((Resource::ResourceType) resourceUrl.port());
+        const QString &editorPath = OpenExternally::selectEditorForResourceType((Resource::ResourceType) resourceUrl.port(), this);
 
         if (!editorPath.isEmpty()) {
             if (OpenExternally::openFile(resourceUrl.toLocalFile(), editorPath)) {
-                const QString &pathname = resourceUrl.toString();
-                emit InsertedFileOpenedExternally(pathname);
+	        const QString bookpath= GetLoadedResource()->GetRelativePath();
+                emit InsertedFileOpenedExternally(bookpath);
             }
         }
     }
@@ -250,8 +264,8 @@ void ImageTab::openWithEditor(int slotnum)
         if (data.isValid() && !editor_path.isEmpty()) {
             const QUrl &resourceUrl = data.toUrl();
             if (OpenExternally::openFile(resourceUrl.toLocalFile(), editor_path)) {
-                const QString &pathname = resourceUrl.toString();
-                emit InsertedFileOpenedExternally(pathname);
+	        const QString bookpath = GetLoadedResource()->GetRelativePath();
+                emit InsertedFileOpenedExternally(bookpath);
             }
         }
     }
@@ -321,6 +335,7 @@ bool ImageTab::SuccessfullySetupContextMenu(const QPoint &point)
 		if (oeaction) {
                     oeaction->setText(aname);
                     oeaction->setData(imageUrl);
+		    oeaction->setEnabled(true);
 		    oeaction->setVisible(true);
 		}
 		i = i + 1;
@@ -362,7 +377,7 @@ void ImageTab::CreateContextMenuActions()
 void ImageTab::ConnectSignalsToSlots()
 {
     connect(m_Resource, SIGNAL(ResourceUpdatedOnDisk()), this, SLOT(RefreshContent()));
-    connect(m_Resource, SIGNAL(Deleted(Resource)), this, SLOT(Close()));
+    connect(m_Resource, SIGNAL(Deleted(const Resource *)), this, SLOT(Close()));
     connect(m_WebView, SIGNAL(customContextMenuRequested(const QPoint &)),  this, SLOT(OpenContextMenu(const QPoint &)));
     connect(m_OpenWith,       SIGNAL(triggered()),   this, SLOT(openWith()));
     connect(m_OpenWithEditor0, SIGNAL(triggered()),  m_openWithMapper, SLOT(map()));
@@ -400,8 +415,7 @@ void ImageTab::Print()
     print_dialog.setWindowTitle(tr("Print %1").arg(GetFilename()));
 
     if (print_dialog.exec() == QDialog::Accepted) {
-        m_WebView->print(&printer);
+        // m_WebView->print(&printer);
     }
 }
 
-
