@@ -1,7 +1,8 @@
 /************************************************************************
 **
-**  Copyright (C) 2018-2019  Kevin B. Hendricks, Stratford, Ontario Canada
-**  Copyright (C) 2009, 2010, 2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
+**  Copyright (C) 2018-2020  Kevin B. Hendricks, Stratford Ontario Canada
+**  Copyright (C) 2019-2020  Doug Massay
+**  Copyright (C) 2009-2011  Strahinja Markovic  <strahinja.markovic@gmail.com>
 **
 **  This file is part of Sigil.
 **
@@ -26,16 +27,17 @@
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
 #include <QtCore/QLibraryInfo>
+#include <QStyleFactory>
 #include <QtCore/QTextCodec>
 #include <QtCore/QThreadPool>
 #include <QtCore/QTranslator>
+#include <QtCore/QStandardPaths>
 #include <QtWidgets/QApplication>
 #include <QtWidgets/QMessageBox>
 #include <QXmlStreamReader>
 #include <QFile>
 #include <QFileInfo>
 #include <QTextStream>
-#include <QDebug>
 
 #include "Misc/PluginDB.h"
 #include "Misc/UILanguage.h"
@@ -60,6 +62,9 @@ static const int RETRY_DELAY_MS = 5;
 #include <QFileDialog>
 #include <QKeySequence>
 #include <QAction>
+#include <QIcon>
+#include "Dialogs/About.h"
+#include "Dialogs/Preferences.h"
 extern void disableWindowTabbing();
 extern void removeMacosSpecificMenuItems();
 #endif
@@ -77,10 +82,31 @@ static MainWindow *GetMainWindow(const QStringList &arguments)
 }
 
 #ifdef Q_OS_MAC
+
+static void AboutDialog()
+{
+    About about;
+    about.exec();
+}
+
+static void PreferencesDialog()
+{
+    Preferences prefers;
+    prefers.exec();
+}
+
+static void AppExit()
+{
+    qApp->closeAllWindows();
+    qApp->quit();
+}
+
 static void file_new()
 {
     MainWindow *w = GetMainWindow(QStringList());
     w->show();
+    w->activateWindow();
+    QCoreApplication::processEvents();
 }
 
 static void file_open()
@@ -98,7 +124,8 @@ static void file_open()
                        "Open File",
                        "~",
                        filter_string,
-                       &default_filter
+		       &default_filter,
+		       QFileDialog::DontUseNativeDialog
                                                    );
 
     if (!filename.isEmpty()) {
@@ -129,6 +156,7 @@ static QIcon GetApplicationIcon()
 void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QString &message)
 {
     QString error_message;
+    QString context_file;
     QString qt_debug_message;
 
     switch (type) {
@@ -151,6 +179,9 @@ void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
         case QtCriticalMsg:
             qt_debug_message = QString("Critical: %1").arg(message.toLatin1().constData());
             error_message = QString(message.toLatin1().constData());
+            if (context.file) context_file = QString(context.file);
+
+	    
 #ifdef Q_OS_WIN32
             // On Windows there is a known issue with the clipboard that results in some copy
             // operations in controls being intermittently blocked. Rather than presenting
@@ -160,7 +191,7 @@ void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
                 QWidget *widget = QApplication::focusWidget();
 
                 if (widget) {
-                    QPlainTextEdit *textEdit = dynamic_cast<QPlainTextEdit *>(widget);
+                    QPlainTextEdit *textEdit = qobject_cast<QPlainTextEdit *>(widget);
 
                     if (textEdit) {
                         QTimer::singleShot(RETRY_DELAY_MS, textEdit, SLOT(copy()));
@@ -168,14 +199,14 @@ void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
                     }
 
                     // Same issue can happen on a QLineEdit / QComboBox
-                    QLineEdit *lineEdit = dynamic_cast<QLineEdit *>(widget);
+                    QLineEdit *lineEdit = qobject_cast<QLineEdit *>(widget);
 
                     if (lineEdit) {
                         QTimer::singleShot(RETRY_DELAY_MS, lineEdit, SLOT(copy()));
                         break;
                     }
 
-                    QComboBox *comboBox = dynamic_cast<QComboBox *>(widget);
+                    QComboBox *comboBox = qobject_cast<QComboBox *>(widget);
 
                     if (comboBox) {
                         QTimer::singleShot(RETRY_DELAY_MS, comboBox->lineEdit(), SLOT(copy()));
@@ -185,7 +216,10 @@ void MessageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
             }
 
 #endif
-            Utility::DisplayExceptionErrorDialog(QString("Critical: %1").arg(error_message));
+            // screen out error messages from inspector / devtools
+            if (!context_file.contains("devtools://devtools")) {
+                Utility::DisplayExceptionErrorDialog(QString("Critical: %1").arg(error_message));
+	    }
             break;
 
         case QtFatalMsg:
@@ -213,6 +247,71 @@ void VerifyPlugins()
 }
 
 
+void setupHighDPI()
+{
+    bool has_env_setting = false;
+    QStringList env_vars;
+    env_vars << "QT_ENABLE_HIGHDPI_SCALING" << "QT_SCALE_FACTOR_ROUNDING_POLICY"
+             << "QT_AUTO_SCREEN_SCALE_FACTOR" << "QT_SCALE_FACTOR"
+             << "QT_SCREEN_SCALE_FACTORS" << "QT_DEVICE_PIXEL_RATIO";
+    foreach(QString v, env_vars) {
+        if (!Utility::GetEnvironmentVar(v).isEmpty()) {
+            has_env_setting = true;
+            break;
+        }
+    }
+
+    SettingsStore ss;
+    int highdpi = ss.highDPI();
+    if (highdpi == 1 || (highdpi == 0 && !has_env_setting)) {
+        // Turning on Automatic High DPI scaling
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, true);
+    } else if (highdpi == 2) {
+        // Turning off Automatic High DPI scaling
+        QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling, false);
+        foreach(QString v, env_vars) {
+            bool irrel = qunsetenv(v.toUtf8().constData());
+        }
+    }
+}
+
+
+QPalette getDarkPalette()
+{
+    // Dark palette for Sigil
+    QPalette darkPalette;
+
+    darkPalette.setColor(QPalette::Window, QColor(53, 53, 53));
+    darkPalette.setColor(QPalette::Disabled, QPalette::Window, QColor(80, 80, 80));
+    darkPalette.setColor(QPalette::WindowText, QColor(238, 238, 238));
+    darkPalette.setColor(QPalette::Disabled, QPalette::WindowText,
+                        QColor(127, 127, 127));
+    darkPalette.setColor(QPalette::Base, QColor(42, 42, 42));
+    darkPalette.setColor(QPalette::Disabled, QPalette::Base, QColor(80, 80, 80));
+    darkPalette.setColor(QPalette::AlternateBase, QColor(66, 66, 66));
+    darkPalette.setColor(QPalette::ToolTipBase, QColor(53, 53, 53));
+    darkPalette.setColor(QPalette::ToolTipText, QColor(238, 238, 238));
+    darkPalette.setColor(QPalette::Text, QColor(238, 238, 238));
+    darkPalette.setColor(QPalette::Disabled, QPalette::Text, QColor(127, 127, 127));
+    darkPalette.setColor(QPalette::Dark, QColor(35, 35, 35));
+    darkPalette.setColor(QPalette::Shadow, QColor(20, 20, 20));
+    darkPalette.setColor(QPalette::Button, QColor(53, 53, 53));
+    darkPalette.setColor(QPalette::ButtonText, QColor(238, 238, 238));
+    darkPalette.setColor(QPalette::Disabled, QPalette::ButtonText,
+                        QColor(127, 127, 127));
+    darkPalette.setColor(QPalette::BrightText, Qt::red);
+    darkPalette.setColor(QPalette::Link, QColor(108, 180, 238));
+    darkPalette.setColor(QPalette::LinkVisited, QColor(108, 180, 238));
+    darkPalette.setColor(QPalette::Highlight, QColor(42, 130, 218));
+    darkPalette.setColor(QPalette::Disabled, QPalette::Highlight, QColor(80, 80, 80));
+    darkPalette.setColor(QPalette::HighlightedText, QColor(238, 238, 238));
+    darkPalette.setColor(QPalette::Disabled, QPalette::HighlightedText,
+                        QColor(127, 127, 127));
+
+    return darkPalette;
+}
+
+
 // Application entry point
 int main(int argc, char *argv[])
 {
@@ -222,6 +321,17 @@ int main(int argc, char *argv[])
     QT_REQUIRE_VERSION(argc, argv, "5.12.3");
 #endif
 
+#if !defined(Q_OS_WIN32) && !defined(Q_OS_MAC)
+// Unset platform theme plugins/styles environment variables immediately
+// when forcing Sigil's own darkmode palette on Linux
+if (!force_sigil_darkmode_palette.isEmpty()) {
+    QStringList env_vars = {"QT_QPA_PLATFORMTHEME", "QT_STYLE_OVERRIDE"};
+    foreach(QString v, env_vars) {
+        bool irrel = qunsetenv(v.toUtf8().constData());
+    }
+}
+#endif
+
 #ifndef QT_DEBUG
     qInstallMessageHandler(MessageHandler);
 #endif
@@ -231,18 +341,29 @@ int main(int argc, char *argv[])
     QCoreApplication::setOrganizationDomain("sigil-ebook.com");
     QCoreApplication::setApplicationName("sigil");
     QCoreApplication::setApplicationVersion(SIGIL_VERSION);
-    
+
+#ifndef Q_OS_MAC
+    setupHighDPI();
+    QCoreApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
+#endif
+
     // many qtbugs related to mixing 32 and 64 bit qt apps when shader disk cache is used
     // Only use if using Qt5.9.0 or higher
 #if QT_VERSION >= 0x050900
     QCoreApplication::setAttribute(Qt::AA_DisableShaderDiskCache);
 #endif
- 
-#if 0   // On recent processors with multiple cores this leads to over 40 threads at times
-        // We prevent Qt from constantly creating and deleting threads.
-        // Using a negative number forces the threads to stay around;
-        // that way, we always have a steady number of threads ready to do work.
-        QThreadPool::globalInstance()->setExpiryTimeout(-1);
+
+    // Disable ? as Sigil does not use QWhatsThis
+#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
+    QCoreApplication::setAttribute(Qt::AA_DisableWindowContextHelpButton);
+#endif
+
+    // On recent processors with multiple cores this leads to over 40 threads at times
+#if 0   
+    // We prevent Qt from constantly creating and deleting threads.
+    // Using a negative number forces the threads to stay around;
+    // that way, we always have a steady number of threads ready to do work.
+    QThreadPool::globalInstance()->setExpiryTimeout(-1);
 #endif
 
     MainApplication app(argc, argv);
@@ -252,6 +373,7 @@ int main(int argc, char *argv[])
     removeMacosSpecificMenuItems();
 #endif
 
+
     // Install an event filter for the application
     // so we can catch OS X's file open events
     // This needs to be done upfront to prevent events from
@@ -279,6 +401,7 @@ int main(int argc, char *argv[])
 
         QTextCodec::setCodecForLocale(QTextCodec::codecForName("utf8"));
         SettingsStore settings;
+
         // Setup the qtbase_ translator and load the translation for the selected language
         QTranslator qtbaseTranslator;
         const QString qm_name_qtbase = QString("qtbase_%1").arg(settings.uiLanguage());
@@ -307,12 +430,71 @@ int main(int argc, char *argv[])
         }
         app.installTranslator(&sigilTranslator);
 
+#ifndef Q_OS_MAC
+#ifndef Q_OS_WIN32
+        // Use platform themes/styles on Linux unless FORCE_SIGIL_DARKMODE_PALETTE is set
+        if (!force_sigil_darkmode_palette.isEmpty()) {
+            // Fusion style is fully dpi aware on Windows/Linux
+            app.setStyle(QStyleFactory::create("fusion"));
+            // qss stylesheet from resources
+            QString dark_styles = Utility::ReadUnicodeTextFile(":/dark/win-dark-style.qss");
+            app.setStyleSheet(dark_styles);
+            app.setPalette(getDarkPalette());
+        }
+#else
+        if (Utility::WindowsShouldUseDarkMode()) {
+            // Fusion style is fully dpi aware on Windows/Linux
+            app.setStyle(QStyleFactory::create("fusion"));
+            // qss stylesheet from resources
+            QString dark_styles = Utility::ReadUnicodeTextFile(":/dark/win-dark-style.qss");
+            app.setStyleSheet(dark_styles);
+            app.setPalette(getDarkPalette());
+        }
+#endif
+#endif
+
+        // Set ui font from preferences after dark theming
+        QFont f = QFont(QApplication::font());
+#ifdef Q_OS_WIN32
+        if (f.family() == "MS Shell Dlg 2" && f.pointSize() == 8) {
+            // Microsoft's recommended UI defaults
+            f.setFamily("Segoe UI");
+            f.setPointSize(9);
+            QApplication::setFont(f);
+        }
+#elif defined(Q_OS_MAC)
+        // Just in case
+#else
+        if (f.family() == "Sans Serif" && f.pointSize() == 9) {
+            f.setPointSize(10);
+            QApplication::setFont(f);
+        }
+#endif
+        settings.setOriginalUIFont(f.toString());
+        if (!settings.uiFont().isEmpty()) {
+            QFont font;
+            if (font.fromString(settings.uiFont()))
+                QApplication::setFont(font);
+        }
+#ifndef Q_OS_MAC
+        // redo on a timer to ensure in all cases
+        if (!settings.uiFont().isEmpty()) {
+            QFont font;
+            if (font.fromString(settings.uiFont())) {
+                QTimer::singleShot(0, [=]() {
+                    QApplication::setFont(font);
+                } );
+            }
+        }
+#endif
+        // End of UI font stuff
+
         // Check for existing qt_styles.qss in Prefs dir and load it if present
         QString qt_stylesheet_path = Utility::DefinePrefsDir() + "/qt_styles.qss";
         QFileInfo QtStylesheetInfo(qt_stylesheet_path);
         if (QtStylesheetInfo.exists() && QtStylesheetInfo.isFile() && QtStylesheetInfo.isReadable()) {
             QString qtstyles = Utility::ReadUnicodeTextFile(qt_stylesheet_path);
-            app.setStyleSheet(qtstyles);
+            app.setStyleSheet(app.styleSheet().append(qtstyles));
         }
 
         // Qt's setCursorFlashTime(msecs) (or the docs) are broken
@@ -321,9 +503,7 @@ int main(int argc, char *argv[])
         // a value which for Mac OS X is hardcoded to 1000 ms
         // This was the only way I could get Qt to disable cursor blinking on a Mac if desired
         if (qEnvironmentVariableIsSet("SIGIL_DISABLE_CURSOR_BLINK")) {
-            // qDebug() << "trying to disable text cursor blinking";
             app.setCursorFlashTime(0);
-            // qDebug() << "cursorFlashTime: " << app.cursorFlashTime();
         }
         // We set the window icon explicitly on Linux.
         // On Windows this is handled by the RC file,
@@ -364,56 +544,91 @@ int main(int argc, char *argv[])
             // Normal startup
 
 #ifdef Q_OS_MAC
-            // Work around QTBUG-62193 and QTBUG-65245 and others where menubar
+            // Try to Work around QTBUG-62193 and QTBUG-65245 and others where menubar
 	    // menu items are lost under File and Sigil menus and where
 	    // Quit menu gets lost when deleting other windows first
 
-	    // Now we Create and show a frameless translucent QMainWindow to hold
-	    // the menubar.  Note: macOS has a single menubar attached at 
-	    // the top of the screen that all mainwindows share.
-
             app.setQuitOnLastWindowClosed(false);
 
-	    Qt::WindowFlags flags = Qt::Window | Qt::FramelessWindowHint;
-	    QMainWindow * basemw = new QMainWindow(NULL, flags);
-	    basemw->setAttribute(Qt::WA_TranslucentBackground, true);
+            // Create a viable Global MacOS QMenuBar
+            QMenuBar *mac_bar = new QMenuBar(0);
 
-            QMenuBar *mac_menu = new QMenuBar(0);
+
+            // Create the Application Menu
+            QMenu *app_menu = new QMenu("Sigil");
+
+            QIcon icon;
+
+            // Quit
+	    QAction* appquit_action = new QAction(QObject::tr("Quit"));
+            appquit_action->setMenuRole(QAction::QuitRole);
+            appquit_action->setShortcut(QKeySequence("Ctrl+Q"));
+            icon = appquit_action->icon();
+	    icon.addFile(QString::fromUtf8(":/main/process-stop_16px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/process-stop_22px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/process-stop_48px.png"));
+            appquit_action->setIcon(icon);
+            QObject::connect(appquit_action, &QAction::triggered, AppExit);
+	    app_menu->addAction(appquit_action);
+
+            // About
+	    QAction* about_action = new QAction(QObject::tr("About"));
+            about_action->setMenuRole(QAction::AboutRole);
+            icon = about_action->icon();
+	    icon.addFile(QString::fromUtf8(":/main/help-browser_16px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/help-browser_22px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/help-browser_48px.png"));
+            about_action->setIcon(icon);
+            QObject::connect(about_action, &QAction::triggered, AboutDialog);
+	    app_menu->addAction(about_action);
+
+            // Preferences
+	    QAction* prefs_action = new QAction(QObject::tr("Preferences"));
+            prefs_action->setMenuRole(QAction::PreferencesRole);
+            QObject::connect(prefs_action, &QAction::triggered, PreferencesDialog);
+	    app_menu->addAction(prefs_action);
+
+            mac_bar->addMenu(app_menu);
+
+            // now create a File Menu
             QMenu *file_menu = new QMenu("File");
 
-	    // New
-            QAction * new_action = new QAction("New");
+            // New
+            QAction * new_action = new QAction(QObject::tr("New"));
             new_action->setShortcut(QKeySequence("Ctrl+N"));
+	    icon = new_action->icon();
+	    icon.addFile(QString::fromUtf8(":/main/document-new_16px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/document-new_22px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/document-new_48px.png"));
+	    new_action->setIcon(icon);
             QObject::connect(new_action, &QAction::triggered, file_new);
-	    file_menu ->addAction(new_action);
+	    file_menu->addAction(new_action);
 
             // Open
-	    QAction* open_action = new QAction("Open");
+	    QAction* open_action = new QAction(QObject::tr("Open"));
             open_action->setShortcut(QKeySequence("Ctrl+O"));
+	    icon = open_action->icon();
+	    icon.addFile(QString::fromUtf8(":/main/document-open_16px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/document-open_22px.png"));
+	    icon.addFile(QString::fromUtf8(":/main/document-open_48px.png"));
+	    open_action->setIcon(icon);
             QObject::connect(open_action, &QAction::triggered, file_open);
-	    file_menu ->addAction(open_action);
+	    file_menu->addAction(open_action);
 
             // Quit - force add of a secondary quit menu to the file menu
-	    QAction* quit_action = new QAction("Quit");
+	    QAction* quit_action = new QAction(QObject::tr("Quit"));
             quit_action->setMenuRole(QAction::NoRole);
             quit_action->setShortcut(QKeySequence("Ctrl+Q"));
             QObject::connect(quit_action, &QAction::triggered, qApp->quit);
-	    file_menu ->addAction(quit_action);
+	    file_menu->addAction(quit_action);
 
-            mac_menu->addMenu(file_menu);
-            
-	    // Application specific quit menu
-	    // according to Qt docs this is the right way to add an application
-	    // quit menu - but it does not work and will still sometimes get lost
-	    mac_menu->addAction("quit");
-
-	    basemw->setMenuBar(mac_menu);
-            basemw->show();
+            mac_bar->addMenu(file_menu);
 #endif
 
             VerifyPlugins();
             MainWindow *widget = GetMainWindow(arguments);
             widget->show();
+            QApplication::setActiveWindow(widget);
             return app.exec();
         }
     } catch (std::exception &e) {
